"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ParserError = void 0;
class ParserError extends Error {
    constructor(message, token) {
        super(message);
        this.token = token;
    }
}
exports.ParserError = ParserError;
function parse(tokens) {
    const tokenIterator = tokens[Symbol.iterator]();
    let currentToken = tokenIterator.next().value;
    let nextToken = tokenIterator.next().value;
    const eatToken = (value) => {
        if (value && value !== currentToken.value) {
            throw new ParserError(`Unexpected token value, expected '${value}', received '${currentToken.value}'`, currentToken);
        }
        let idx = currentToken.startIdx;
        currentToken = nextToken;
        nextToken = tokenIterator.next().value;
        return idx;
    };
    const eatText = () => {
        checkTypes("text");
        let val = currentToken.value;
        let idx = currentToken.startIdx;
        eatToken();
        return [val, idx];
    };
    const eatNumber = () => {
        checkTypes("number");
        let val = Number(currentToken.value);
        if (Number.isNaN(val)) {
            throw new ParserError(`Value cannot be parsed as number! ${currentToken.value}`, currentToken);
        }
        eatToken();
        return val;
    };
    const checkTypes = (...types) => {
        if (types.indexOf(currentToken.type) < 0) {
            throw new ParserError(`Unexpected token value, expected ${types.join(" | ")}, received '${currentToken.value}'`, currentToken);
        }
    };
    const parsePathStatement = () => {
        const segments = [];
        const idx = currentToken.startIdx;
        let next = currentToken.type === "slash";
        while (next) {
            eatToken("/");
            if (currentToken.type === "curly_open" && nextToken.type === "text") {
                eatToken("{");
                const [name] = eatText();
                segments.push({
                    type: "variable",
                    name,
                });
                eatToken("}");
            }
            else if (currentToken.type === "text") {
                const [name] = eatText();
                segments.push(name);
            }
            next = currentToken.type === "slash";
        }
        return {
            type: "path",
            idx,
            segments,
        };
    };
    const parseValue = () => {
        const idx = currentToken.startIdx;
        let isTrue = false;
        let isFalse = false;
        let isNull = false;
        let isVariable = false;
        let isNumber = false;
        let isString = false;
        let value = undefined;
        if (currentToken.type === "keyword") {
            if (currentToken.value === "true")
                isTrue = true;
            else if (currentToken.value === "false")
                isFalse = true;
            else if (currentToken.value === "null")
                isNull = true;
            else {
                throw new ParserError(`Invalid keyword at this position ${currentToken.value}`, currentToken);
            }
            eatToken();
        }
        else if (currentToken.type === "string") {
            isString = true;
            value = currentToken.value.slice(1, currentToken.value.length - 1);
            eatToken();
        }
        else if (currentToken.type === "number") {
            isNumber = true;
            value = eatNumber();
        }
        else if (currentToken.type === "text") {
            isVariable = true;
            [value] = eatText();
        }
        else {
            throw new ParserError(`Expected value got ${currentToken.type}`, currentToken);
        }
        return {
            type: "value",
            isFalse,
            isNull,
            isNumber,
            isString,
            isTrue,
            isVariable,
            value,
            idx,
        };
    };
    const parseCondition = () => {
        // let running = true;
        let res;
        let left;
        // while (running) {
        const idx = currentToken.startIdx;
        if (!left) {
            if (currentToken.type === "bracket_open") {
                eatToken("(");
                left = parseCondition();
                eatToken(")");
            }
            else {
                left = parseValue();
            }
        }
        if (currentToken.type === "comparison_operator") {
            const operator = currentToken.value;
            eatToken();
            let right;
            let ct = currentToken; //Quick hack because of TypeScript
            if (ct.type === "bracket_open") {
                eatToken("(");
                right = parseCondition();
                eatToken(")");
            }
            else {
                right = parseValue();
            }
            res = {
                type: "expression",
                left,
                right,
                operator: operator,
                idx,
            };
        }
        else if (currentToken.type === "logic_operator") {
            const operator = currentToken.value;
            eatToken();
            const right = parseCondition();
            res = {
                type: "expression",
                left,
                operator: operator,
                right,
                idx,
            };
        }
        else {
            res = left;
        }
        // let ct = currentToken;
        // if (
        //    ct.type === "comparison_operator" ||
        //    ct.type === "logic_operator"
        // ) {
        //    left = res;
        // } else {
        //    running = false;
        // }
        // }
        return res;
    };
    const parsePermissionStatement = () => {
        const idx = eatToken("allow");
        const operations = [];
        let next = currentToken.type !== "colon";
        while (next) {
            const [operation] = eatText();
            operations.push(operation);
            if (currentToken.type === "comma") {
                next = true;
                eatToken(",");
            }
            else {
                next = false;
            }
        }
        eatToken(":");
        eatToken("if");
        const condition = parseCondition();
        eatToken(";");
        return {
            type: "permission",
            idx,
            operations,
            condition,
        };
    };
    const parseMatchStatement = () => {
        const idx = eatToken("match");
        const path = parsePathStatement();
        eatToken("{");
        const matches = [];
        const permissions = [];
        while (currentToken.type !== "curly_close") {
            if (currentToken.value === "match") {
                matches.push(parseMatchStatement());
            }
            else if (currentToken.value === "allow") {
                permissions.push(parsePermissionStatement());
            }
            else {
                throw new ParserError(`Unexpected token value, expected 'match' or 'allow', received '${currentToken.value}'`, currentToken);
            }
        }
        eatToken("}");
        return {
            type: "match",
            path,
            idx,
            matches,
            rules: permissions,
        };
    };
    const parseServiceStatement = () => {
        const idx = eatToken("service");
        let [name] = eatText();
        eatToken("{");
        const matches = [];
        while (currentToken.value === "match") {
            matches.push(parseMatchStatement());
        }
        eatToken("}");
        return {
            type: "service",
            name: name,
            idx,
            matches,
        };
    };
    const nodes = [];
    while (currentToken) {
        nodes.push(parseServiceStatement());
    }
    return nodes;
}
exports.default = parse;
//# sourceMappingURL=parser.js.map